@@ -5,7 +5,11 @@
 #include <QByteArray>
 #include <QString>
 
+struct wl_registry;
+
 QT_BEGIN_NAMESPACE
+QT_WARNING_PUSH
+QT_WARNING_DISABLE_GCC("-Wmissing-field-initializers")
 
 #if !defined(Q_WAYLAND_CLIENT_SERVER_BUFFER_EXTENSION_EXPORT)
 #  if defined(QT_SHARED)
@@ -42,6 +46,7 @@ namespace QtWayland {
     };
 }
 
+QT_WARNING_POP
 QT_END_NAMESPACE
 
 #endif
