@@ -5,7 +5,11 @@
 #include <QByteArray>
 #include <QString>
 
+struct wl_registry;
+
 QT_BEGIN_NAMESPACE
+QT_WARNING_PUSH
+QT_WARNING_DISABLE_GCC("-Wmissing-field-initializers")
 
 #if !defined(Q_WAYLAND_CLIENT_WAYLAND_EXPORT)
 #  if defined(QT_SHARED)
@@ -168,64 +172,64 @@ namespace QtWayland {
         };
 
         enum format {
-            format_argb8888 = 0, // 32-bit ARGB format
-            format_xrgb8888 = 1, // 32-bit RGB format
-            format_c8 = 0x20203843,
-            format_rgb332 = 0x38424752,
-            format_bgr233 = 0x38524742,
-            format_xrgb4444 = 0x32315258,
-            format_xbgr4444 = 0x32314258,
-            format_rgbx4444 = 0x32315852,
-            format_bgrx4444 = 0x32315842,
-            format_argb4444 = 0x32315241,
-            format_abgr4444 = 0x32314241,
-            format_rgba4444 = 0x32314152,
-            format_bgra4444 = 0x32314142,
-            format_xrgb1555 = 0x35315258,
-            format_xbgr1555 = 0x35314258,
-            format_rgbx5551 = 0x35315852,
-            format_bgrx5551 = 0x35315842,
-            format_argb1555 = 0x35315241,
-            format_abgr1555 = 0x35314241,
-            format_rgba5551 = 0x35314152,
-            format_bgra5551 = 0x35314142,
-            format_rgb565 = 0x36314752,
-            format_bgr565 = 0x36314742,
-            format_rgb888 = 0x34324752,
-            format_bgr888 = 0x34324742,
-            format_xbgr8888 = 0x34324258,
-            format_rgbx8888 = 0x34325852,
-            format_bgrx8888 = 0x34325842,
-            format_abgr8888 = 0x34324241,
-            format_rgba8888 = 0x34324152,
-            format_bgra8888 = 0x34324142,
-            format_xrgb2101010 = 0x30335258,
-            format_xbgr2101010 = 0x30334258,
-            format_rgbx1010102 = 0x30335852,
-            format_bgrx1010102 = 0x30335842,
-            format_argb2101010 = 0x30335241,
-            format_abgr2101010 = 0x30334241,
-            format_rgba1010102 = 0x30334152,
-            format_bgra1010102 = 0x30334142,
-            format_yuyv = 0x56595559,
-            format_yvyu = 0x55595659,
-            format_uyvy = 0x59565955,
-            format_vyuy = 0x59555956,
-            format_ayuv = 0x56555941,
-            format_nv12 = 0x3231564e,
-            format_nv21 = 0x3132564e,
-            format_nv16 = 0x3631564e,
-            format_nv61 = 0x3136564e,
-            format_yuv410 = 0x39565559,
-            format_yvu410 = 0x39555659,
-            format_yuv411 = 0x31315559,
-            format_yvu411 = 0x31315659,
-            format_yuv420 = 0x32315559,
-            format_yvu420 = 0x32315659,
-            format_yuv422 = 0x36315559,
-            format_yvu422 = 0x36315659,
-            format_yuv444 = 0x34325559,
-            format_yvu444 = 0x34325659
+            format_argb8888 = 0, // 32-bit ARGB format, [31:0] A:R:G:B 8:8:8:8 little endian
+            format_xrgb8888 = 1, // 32-bit RGB format, [31:0] x:R:G:B 8:8:8:8 little endian
+            format_c8 = 0x20203843, // 8-bit color index format, [7:0] C
+            format_rgb332 = 0x38424752, // 8-bit RGB format, [7:0] R:G:B 3:3:2
+            format_bgr233 = 0x38524742, // 8-bit BGR format, [7:0] B:G:R 2:3:3
+            format_xrgb4444 = 0x32315258, // 16-bit xRGB format, [15:0] x:R:G:B 4:4:4:4 little endian
+            format_xbgr4444 = 0x32314258, // 16-bit xBGR format, [15:0] x:B:G:R 4:4:4:4 little endian
+            format_rgbx4444 = 0x32315852, // 16-bit RGBx format, [15:0] R:G:B:x 4:4:4:4 little endian
+            format_bgrx4444 = 0x32315842, // 16-bit BGRx format, [15:0] B:G:R:x 4:4:4:4 little endian
+            format_argb4444 = 0x32315241, // 16-bit ARGB format, [15:0] A:R:G:B 4:4:4:4 little endian
+            format_abgr4444 = 0x32314241, // 16-bit ABGR format, [15:0] A:B:G:R 4:4:4:4 little endian
+            format_rgba4444 = 0x32314152, // 16-bit RBGA format, [15:0] R:G:B:A 4:4:4:4 little endian
+            format_bgra4444 = 0x32314142, // 16-bit BGRA format, [15:0] B:G:R:A 4:4:4:4 little endian
+            format_xrgb1555 = 0x35315258, // 16-bit xRGB format, [15:0] x:R:G:B 1:5:5:5 little endian
+            format_xbgr1555 = 0x35314258, // 16-bit xBGR 1555 format, [15:0] x:B:G:R 1:5:5:5 little endian
+            format_rgbx5551 = 0x35315852, // 16-bit RGBx 5551 format, [15:0] R:G:B:x 5:5:5:1 little endian
+            format_bgrx5551 = 0x35315842, // 16-bit BGRx 5551 format, [15:0] B:G:R:x 5:5:5:1 little endian
+            format_argb1555 = 0x35315241, // 16-bit ARGB 1555 format, [15:0] A:R:G:B 1:5:5:5 little endian
+            format_abgr1555 = 0x35314241, // 16-bit ABGR 1555 format, [15:0] A:B:G:R 1:5:5:5 little endian
+            format_rgba5551 = 0x35314152, // 16-bit RGBA 5551 format, [15:0] R:G:B:A 5:5:5:1 little endian
+            format_bgra5551 = 0x35314142, // 16-bit BGRA 5551 format, [15:0] B:G:R:A 5:5:5:1 little endian
+            format_rgb565 = 0x36314752, // 16-bit RGB 565 format, [15:0] R:G:B 5:6:5 little endian
+            format_bgr565 = 0x36314742, // 16-bit BGR 565 format, [15:0] B:G:R 5:6:5 little endian
+            format_rgb888 = 0x34324752, // 24-bit RGB format, [23:0] R:G:B little endian
+            format_bgr888 = 0x34324742, // 24-bit BGR format, [23:0] B:G:R little endian
+            format_xbgr8888 = 0x34324258, // 32-bit xBGR format, [31:0] x:B:G:R 8:8:8:8 little endian
+            format_rgbx8888 = 0x34325852, // 32-bit RGBx format, [31:0] R:G:B:x 8:8:8:8 little endian
+            format_bgrx8888 = 0x34325842, // 32-bit BGRx format, [31:0] B:G:R:x 8:8:8:8 little endian
+            format_abgr8888 = 0x34324241, // 32-bit ABGR format, [31:0] A:B:G:R 8:8:8:8 little endian
+            format_rgba8888 = 0x34324152, // 32-bit RGBA format, [31:0] R:G:B:A 8:8:8:8 little endian
+            format_bgra8888 = 0x34324142, // 32-bit BGRA format, [31:0] B:G:R:A 8:8:8:8 little endian
+            format_xrgb2101010 = 0x30335258, // 32-bit xRGB format, [31:0] x:R:G:B 2:10:10:10 little endian
+            format_xbgr2101010 = 0x30334258, // 32-bit xBGR format, [31:0] x:B:G:R 2:10:10:10 little endian
+            format_rgbx1010102 = 0x30335852, // 32-bit RGBx format, [31:0] R:G:B:x 10:10:10:2 little endian
+            format_bgrx1010102 = 0x30335842, // 32-bit BGRx format, [31:0] B:G:R:x 10:10:10:2 little endian
+            format_argb2101010 = 0x30335241, // 32-bit ARGB format, [31:0] A:R:G:B 2:10:10:10 little endian
+            format_abgr2101010 = 0x30334241, // 32-bit ABGR format, [31:0] A:B:G:R 2:10:10:10 little endian
+            format_rgba1010102 = 0x30334152, // 32-bit RGBA format, [31:0] R:G:B:A 10:10:10:2 little endian
+            format_bgra1010102 = 0x30334142, // 32-bit BGRA format, [31:0] B:G:R:A 10:10:10:2 little endian
+            format_yuyv = 0x56595559, // packed YCbCr format, [31:0] Cr0:Y1:Cb0:Y0 8:8:8:8 little endian
+            format_yvyu = 0x55595659, // packed YCbCr format, [31:0] Cb0:Y1:Cr0:Y0 8:8:8:8 little endian
+            format_uyvy = 0x59565955, // packed YCbCr format, [31:0] Y1:Cr0:Y0:Cb0 8:8:8:8 little endian
+            format_vyuy = 0x59555956, // packed YCbCr format, [31:0] Y1:Cb0:Y0:Cr0 8:8:8:8 little endian
+            format_ayuv = 0x56555941, // packed AYCbCr format, [31:0] A:Y:Cb:Cr 8:8:8:8 little endian
+            format_nv12 = 0x3231564e, // 2 plane YCbCr Cr:Cb format, 2x2 subsampled Cr:Cb plane
+            format_nv21 = 0x3132564e, // 2 plane YCbCr Cb:Cr format, 2x2 subsampled Cb:Cr plane
+            format_nv16 = 0x3631564e, // 2 plane YCbCr Cr:Cb format, 2x1 subsampled Cr:Cb plane
+            format_nv61 = 0x3136564e, // 2 plane YCbCr Cb:Cr format, 2x1 subsampled Cb:Cr plane
+            format_yuv410 = 0x39565559, // 3 plane YCbCr format, 4x4 subsampled Cb (1) and Cr (2) planes
+            format_yvu410 = 0x39555659, // 3 plane YCbCr format, 4x4 subsampled Cr (1) and Cb (2) planes
+            format_yuv411 = 0x31315559, // 3 plane YCbCr format, 4x1 subsampled Cb (1) and Cr (2) planes
+            format_yvu411 = 0x31315659, // 3 plane YCbCr format, 4x1 subsampled Cr (1) and Cb (2) planes
+            format_yuv420 = 0x32315559, // 3 plane YCbCr format, 2x2 subsampled Cb (1) and Cr (2) planes
+            format_yvu420 = 0x32315659, // 3 plane YCbCr format, 2x2 subsampled Cr (1) and Cb (2) planes
+            format_yuv422 = 0x36315559, // 3 plane YCbCr format, 2x1 subsampled Cb (1) and Cr (2) planes
+            format_yvu422 = 0x36315659, // 3 plane YCbCr format, 2x1 subsampled Cr (1) and Cb (2) planes
+            format_yuv444 = 0x34325559, // 3 plane YCbCr format, non-subsampled Cb (1) and Cr (2) planes
+            format_yvu444 = 0x34325659 // 3 plane YCbCr format, non-subsampled Cr (1) and Cb (2) planes
         };
 
         struct ::wl_shm_pool *create_pool(int32_t fd, int32_t size);
@@ -295,12 +299,23 @@ namespace QtWayland {
 
         static const struct ::wl_interface *interface();
 
+        enum error {
+            error_invalid_finish = 0, // finish request was called untimely
+            error_invalid_action_mask = 1, // action mask contains invalid values
+            error_invalid_action = 2, // action argument has an invalid value
+            error_invalid_offer = 3 // offer doesn't accept this request
+        };
+
         void accept(uint32_t serial, const QString &mime_type);
         void receive(const QString &mime_type, int32_t fd);
         void destroy();
+        void finish();
+        void set_actions(uint32_t dnd_actions, uint32_t preferred_action);
 
     protected:
         virtual void data_offer_offer(const QString &mime_type);
+        virtual void data_offer_source_actions(uint32_t source_actions);
+        virtual void data_offer_action(uint32_t dnd_action);
 
     private:
         void init_listener();
@@ -309,6 +324,14 @@ namespace QtWayland {
             void *data,
             struct ::wl_data_offer *object,
             const char *mime_type);
+        static void handle_source_actions(
+            void *data,
+            struct ::wl_data_offer *object,
+            uint32_t source_actions);
+        static void handle_action(
+            void *data,
+            struct ::wl_data_offer *object,
+            uint32_t dnd_action);
         struct ::wl_data_offer *m_wl_data_offer;
     };
 
@@ -331,13 +354,22 @@ namespace QtWayland {
 
         static const struct ::wl_interface *interface();
 
+        enum error {
+            error_invalid_action_mask = 0, // action mask contains invalid values
+            error_invalid_source = 1 // source doesn't accept this request
+        };
+
         void offer(const QString &mime_type);
         void destroy();
+        void set_actions(uint32_t dnd_actions);
 
     protected:
         virtual void data_source_target(const QString &mime_type);
         virtual void data_source_send(const QString &mime_type, int32_t fd);
         virtual void data_source_cancelled();
+        virtual void data_source_dnd_drop_performed();
+        virtual void data_source_dnd_finished();
+        virtual void data_source_action(uint32_t dnd_action);
 
     private:
         void init_listener();
@@ -354,6 +386,16 @@ namespace QtWayland {
         static void handle_cancelled(
             void *data,
             struct ::wl_data_source *object);
+        static void handle_dnd_drop_performed(
+            void *data,
+            struct ::wl_data_source *object);
+        static void handle_dnd_finished(
+            void *data,
+            struct ::wl_data_source *object);
+        static void handle_action(
+            void *data,
+            struct ::wl_data_source *object,
+            uint32_t dnd_action);
         struct ::wl_data_source *m_wl_data_source;
     };
 
@@ -376,8 +418,13 @@ namespace QtWayland {
 
         static const struct ::wl_interface *interface();
 
+        enum error {
+            error_role = 0 // given wl_surface has another role
+        };
+
         void start_drag(struct ::wl_data_source *source, struct ::wl_surface *origin, struct ::wl_surface *icon, uint32_t serial);
         void set_selection(struct ::wl_data_source *source, uint32_t serial);
+        void release();
 
     protected:
         virtual void data_device_data_offer(struct ::wl_data_offer *id);
@@ -440,6 +487,13 @@ namespace QtWayland {
 
         static const struct ::wl_interface *interface();
 
+        enum dnd_action {
+            dnd_action_none = 0, // no action
+            dnd_action_copy = 1, // copy action
+            dnd_action_move = 2, // move action
+            dnd_action_ask = 4 // ask action
+        };
+
         struct ::wl_data_source *create_data_source();
         struct ::wl_data_device *get_data_device(struct ::wl_seat *seat);
 
@@ -466,6 +520,10 @@ namespace QtWayland {
 
         static const struct ::wl_interface *interface();
 
+        enum error {
+            error_role = 0 // given wl_surface has another role
+        };
+
         struct ::wl_shell_surface *get_shell_surface(struct ::wl_surface *surface);
 
     private:
@@ -492,15 +550,15 @@ namespace QtWayland {
         static const struct ::wl_interface *interface();
 
         enum resize {
-            resize_none = 0,
-            resize_top = 1,
-            resize_bottom = 2,
-            resize_left = 4,
-            resize_top_left = 5,
-            resize_bottom_left = 6,
-            resize_right = 8,
-            resize_top_right = 9,
-            resize_bottom_right = 10
+            resize_none = 0, // no edge
+            resize_top = 1, // top edge
+            resize_bottom = 2, // bottom edge
+            resize_left = 4, // left edge
+            resize_top_left = 5, // top and left edges
+            resize_bottom_left = 6, // bottom and left edges
+            resize_right = 8, // right edge
+            resize_top_right = 9, // top and right edges
+            resize_bottom_right = 10 // bottom and right edges
         };
 
         enum transient {
@@ -582,6 +640,7 @@ namespace QtWayland {
         void commit();
         void set_buffer_transform(int32_t transform);
         void set_buffer_scale(int32_t scale);
+        void damage_buffer(int32_t x, int32_t y, int32_t width, int32_t height);
 
     protected:
         virtual void surface_enter(struct ::wl_output *output);
@@ -621,14 +680,15 @@ namespace QtWayland {
         static const struct ::wl_interface *interface();
 
         enum capability {
-            capability_pointer = 1, // The seat has pointer devices
-            capability_keyboard = 2, // The seat has one or more keyboards
-            capability_touch = 4 // The seat has touch devices
+            capability_pointer = 1, // the seat has pointer devices
+            capability_keyboard = 2, // the seat has one or more keyboards
+            capability_touch = 4 // the seat has touch devices
         };
 
         struct ::wl_pointer *get_pointer();
         struct ::wl_keyboard *get_keyboard();
         struct ::wl_touch *get_touch();
+        void release();
 
     protected:
         virtual void seat_capabilities(uint32_t capabilities);
@@ -667,14 +727,25 @@ namespace QtWayland {
 
         static const struct ::wl_interface *interface();
 
+        enum error {
+            error_role = 0 // given wl_surface has another role
+        };
+
         enum button_state {
-            button_state_released = 0, // The button is not pressed
-            button_state_pressed = 1 // The button is pressed
+            button_state_released = 0, // the button is not pressed
+            button_state_pressed = 1 // the button is pressed
         };
 
         enum axis {
-            axis_vertical_scroll = 0,
-            axis_horizontal_scroll = 1
+            axis_vertical_scroll = 0, // vertical axis
+            axis_horizontal_scroll = 1 // horizontal axis
+        };
+
+        enum axis_source {
+            axis_source_wheel = 0, // a physical wheel rotation
+            axis_source_finger = 1, // finger on a touch surface
+            axis_source_continuous = 2, // continuous coordinate space
+            axis_source_wheel_tilt = 3 // a physical wheel tilt
         };
 
         void set_cursor(uint32_t serial, struct ::wl_surface *surface, int32_t hotspot_x, int32_t hotspot_y);
@@ -686,6 +757,10 @@ namespace QtWayland {
         virtual void pointer_motion(uint32_t time, wl_fixed_t surface_x, wl_fixed_t surface_y);
         virtual void pointer_button(uint32_t serial, uint32_t time, uint32_t button, uint32_t state);
         virtual void pointer_axis(uint32_t time, uint32_t axis, wl_fixed_t value);
+        virtual void pointer_frame();
+        virtual void pointer_axis_source(uint32_t axis_source);
+        virtual void pointer_axis_stop(uint32_t time, uint32_t axis);
+        virtual void pointer_axis_discrete(uint32_t axis, int32_t discrete);
 
     private:
         void init_listener();
@@ -721,6 +796,23 @@ namespace QtWayland {
             uint32_t time,
             uint32_t axis,
             wl_fixed_t value);
+        static void handle_frame(
+            void *data,
+            struct ::wl_pointer *object);
+        static void handle_axis_source(
+            void *data,
+            struct ::wl_pointer *object,
+            uint32_t axis_source);
+        static void handle_axis_stop(
+            void *data,
+            struct ::wl_pointer *object,
+            uint32_t time,
+            uint32_t axis);
+        static void handle_axis_discrete(
+            void *data,
+            struct ::wl_pointer *object,
+            uint32_t axis,
+            int32_t discrete);
         struct ::wl_pointer *m_wl_pointer;
     };
 
@@ -833,6 +925,8 @@ namespace QtWayland {
         virtual void touch_motion(uint32_t time, int32_t id, wl_fixed_t x, wl_fixed_t y);
         virtual void touch_frame();
         virtual void touch_cancel();
+        virtual void touch_shape(int32_t id, wl_fixed_t major, wl_fixed_t minor);
+        virtual void touch_orientation(int32_t id, wl_fixed_t orientation);
 
     private:
         void init_listener();
@@ -865,6 +959,17 @@ namespace QtWayland {
         static void handle_cancel(
             void *data,
             struct ::wl_touch *object);
+        static void handle_shape(
+            void *data,
+            struct ::wl_touch *object,
+            int32_t id,
+            wl_fixed_t major,
+            wl_fixed_t minor);
+        static void handle_orientation(
+            void *data,
+            struct ::wl_touch *object,
+            int32_t id,
+            wl_fixed_t orientation);
         struct ::wl_touch *m_wl_touch;
     };
 
@@ -888,23 +993,23 @@ namespace QtWayland {
         static const struct ::wl_interface *interface();
 
         enum subpixel {
-            subpixel_unknown = 0,
-            subpixel_none = 1,
-            subpixel_horizontal_rgb = 2,
-            subpixel_horizontal_bgr = 3,
-            subpixel_vertical_rgb = 4,
-            subpixel_vertical_bgr = 5
+            subpixel_unknown = 0, // unknown geometry
+            subpixel_none = 1, // no geometry
+            subpixel_horizontal_rgb = 2, // horizontal RGB
+            subpixel_horizontal_bgr = 3, // horizontal BGR
+            subpixel_vertical_rgb = 4, // vertical RGB
+            subpixel_vertical_bgr = 5 // vertical BGR
         };
 
         enum transform {
-            transform_normal = 0,
-            transform_90 = 1,
-            transform_180 = 2,
-            transform_270 = 3,
-            transform_flipped = 4,
-            transform_flipped_90 = 5,
-            transform_flipped_180 = 6,
-            transform_flipped_270 = 7
+            transform_normal = 0, // no transform
+            transform_90 = 1, // 90 degrees counter-clockwise
+            transform_180 = 2, // 180 degrees counter-clockwise
+            transform_270 = 3, // 270 degrees counter-clockwise
+            transform_flipped = 4, // 180 degree flip around a vertical axis
+            transform_flipped_90 = 5, // flip and rotate 90 degrees counter-clockwise
+            transform_flipped_180 = 6, // flip and rotate 180 degrees counter-clockwise
+            transform_flipped_270 = 7 // flip and rotate 270 degrees counter-clockwise
         };
 
         enum mode {
@@ -912,6 +1017,8 @@ namespace QtWayland {
             mode_preferred = 0x2 // indicates this is the preferred mode
         };
 
+        void release();
+
     protected:
         virtual void output_geometry(int32_t x, int32_t y, int32_t physical_width, int32_t physical_height, int32_t subpixel, const QString &make, const QString &model, int32_t transform);
         virtual void output_mode(uint32_t flags, int32_t width, int32_t height, int32_t refresh);
@@ -1041,6 +1148,7 @@ namespace QtWayland {
     };
 }
 
+QT_WARNING_POP
 QT_END_NAMESPACE
 
 #endif
